"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.loadConfig = void 0;
const tslib_1 = require("tslib");
const utils_fs_1 = require("@ionic/utils-fs");
const debug_1 = tslib_1.__importDefault(require("debug"));
const path_1 = require("path");
const colors_1 = tslib_1.__importDefault(require("./colors"));
const log_1 = require("./log");
const fn_1 = require("./util/fn");
const node_1 = require("./util/node");
const debug = debug_1.default('capacitor:config');
async function loadConfig() {
    var _a, _b, _c, _d, _e;
    const appRootDir = process.cwd();
    const cliRootDir = path_1.dirname(__dirname);
    const conf = await loadExtConfig(appRootDir);
    const appId = (_a = conf.extConfig.appId) !== null && _a !== void 0 ? _a : '';
    const appName = (_b = conf.extConfig.appName) !== null && _b !== void 0 ? _b : '';
    const webDir = (_c = conf.extConfig.webDir) !== null && _c !== void 0 ? _c : 'www';
    const cli = await loadCLIConfig(cliRootDir);
    const config = {
        android: await loadAndroidConfig(appRootDir, conf.extConfig, cli),
        ios: await loadIOSConfig(appRootDir, conf.extConfig),
        web: await loadWebConfig(appRootDir, webDir),
        cli,
        app: {
            rootDir: appRootDir,
            appId,
            appName,
            webDir,
            webDirAbs: path_1.resolve(appRootDir, webDir),
            package: (_d = (await fn_1.tryFn(utils_fs_1.readJSON, path_1.resolve(appRootDir, 'package.json')))) !== null && _d !== void 0 ? _d : {
                name: appName,
                version: '1.0.0',
            },
            ...conf,
            bundledWebRuntime: (_e = conf.extConfig.bundledWebRuntime) !== null && _e !== void 0 ? _e : false,
        },
    };
    debug('config: %O', config);
    return config;
}
exports.loadConfig = loadConfig;
async function loadExtConfigTS(rootDir, extConfigName, extConfigFilePath) {
    var _a;
    try {
        const tsPath = node_1.resolveNode(rootDir, 'typescript');
        if (!tsPath) {
            log_1.logFatal('Could not find installation of TypeScript.\n' +
                `To use ${colors_1.default.strong(extConfigName)} files, you must install TypeScript in your project, e.g. w/ ${colors_1.default.input('npm install -D typescript')}`);
        }
        const ts = require(tsPath); // eslint-disable-line @typescript-eslint/no-var-requires
        return {
            extConfigType: 'ts',
            extConfigName,
            extConfigFilePath: extConfigFilePath,
            extConfig: node_1.requireTS(ts, extConfigFilePath),
        };
    }
    catch (e) {
        log_1.logFatal(`Parsing ${colors_1.default.strong(extConfigName)} failed.\n\n${(_a = e.stack) !== null && _a !== void 0 ? _a : e}`);
    }
}
async function loadExtConfigJS(rootDir, extConfigName, extConfigFilePath) {
    var _a;
    try {
        return {
            extConfigType: 'js',
            extConfigName,
            extConfigFilePath: extConfigFilePath,
            extConfig: require(extConfigFilePath),
        };
    }
    catch (e) {
        log_1.logFatal(`Parsing ${colors_1.default.strong(extConfigName)} failed.\n\n${(_a = e.stack) !== null && _a !== void 0 ? _a : e}`);
    }
}
async function loadExtConfig(rootDir) {
    var _a;
    const extConfigNameTS = 'capacitor.config.ts';
    const extConfigFilePathTS = path_1.resolve(rootDir, extConfigNameTS);
    if (await utils_fs_1.pathExists(extConfigFilePathTS)) {
        return loadExtConfigTS(rootDir, extConfigNameTS, extConfigFilePathTS);
    }
    const extConfigNameJS = 'capacitor.config.js';
    const extConfigFilePathJS = path_1.resolve(rootDir, extConfigNameJS);
    if (await utils_fs_1.pathExists(extConfigFilePathJS)) {
        return loadExtConfigJS(rootDir, extConfigNameJS, extConfigFilePathJS);
    }
    const extConfigName = 'capacitor.config.json';
    const extConfigFilePath = path_1.resolve(rootDir, extConfigName);
    return {
        extConfigType: 'json',
        extConfigName,
        extConfigFilePath: extConfigFilePath,
        extConfig: (_a = (await fn_1.tryFn(utils_fs_1.readJSON, extConfigFilePath))) !== null && _a !== void 0 ? _a : {},
    };
}
async function loadCLIConfig(rootDir) {
    const assetsDir = 'assets';
    const assetsDirAbs = path_1.join(rootDir, assetsDir);
    const iosPlatformTemplateArchive = 'ios-template.tar.gz';
    const iosCordovaPluginsTemplateArchive = 'capacitor-cordova-ios-plugins.tar.gz';
    const androidPlatformTemplateArchive = 'android-template.tar.gz';
    const androidCordovaPluginsTemplateArchive = 'capacitor-cordova-android-plugins.tar.gz';
    return {
        rootDir,
        assetsDir,
        assetsDirAbs,
        assets: {
            ios: {
                platformTemplateArchive: iosPlatformTemplateArchive,
                platformTemplateArchiveAbs: path_1.resolve(assetsDirAbs, iosPlatformTemplateArchive),
                cordovaPluginsTemplateArchive: iosCordovaPluginsTemplateArchive,
                cordovaPluginsTemplateArchiveAbs: path_1.resolve(assetsDirAbs, iosCordovaPluginsTemplateArchive),
            },
            android: {
                platformTemplateArchive: androidPlatformTemplateArchive,
                platformTemplateArchiveAbs: path_1.resolve(assetsDirAbs, androidPlatformTemplateArchive),
                cordovaPluginsTemplateArchive: androidCordovaPluginsTemplateArchive,
                cordovaPluginsTemplateArchiveAbs: path_1.resolve(assetsDirAbs, androidCordovaPluginsTemplateArchive),
            },
        },
        package: await utils_fs_1.readJSON(path_1.resolve(rootDir, 'package.json')),
        os: determineOS(process.platform),
    };
}
async function loadAndroidConfig(rootDir, extConfig, cliConfig) {
    var _a, _b;
    const name = 'android';
    const platformDir = (_b = (_a = extConfig.android) === null || _a === void 0 ? void 0 : _a.path) !== null && _b !== void 0 ? _b : 'android';
    const platformDirAbs = path_1.resolve(rootDir, platformDir);
    const appDir = 'app';
    const srcDir = `${appDir}/src`;
    const srcMainDir = `${srcDir}/main`;
    const assetsDir = `${srcMainDir}/assets`;
    const webDir = `${assetsDir}/public`;
    const resDir = `${srcMainDir}/res`;
    const buildOutputDir = `${appDir}/build/outputs/apk/debug`;
    const cordovaPluginsDir = 'capacitor-cordova-android-plugins';
    const studioPath = await determineAndroidStudioPath(cliConfig.os);
    return {
        name,
        minVersion: '21',
        studioPath,
        platformDir,
        platformDirAbs,
        cordovaPluginsDir,
        cordovaPluginsDirAbs: path_1.resolve(platformDirAbs, cordovaPluginsDir),
        appDir,
        appDirAbs: path_1.resolve(platformDirAbs, appDir),
        srcDir,
        srcDirAbs: path_1.resolve(platformDirAbs, srcDir),
        srcMainDir,
        srcMainDirAbs: path_1.resolve(platformDirAbs, srcMainDir),
        assetsDir,
        assetsDirAbs: path_1.resolve(platformDirAbs, assetsDir),
        webDir,
        webDirAbs: path_1.resolve(platformDirAbs, webDir),
        resDir,
        resDirAbs: path_1.resolve(platformDirAbs, resDir),
        buildOutputDir,
        buildOutputDirAbs: path_1.resolve(platformDirAbs, buildOutputDir),
    };
}
async function loadIOSConfig(rootDir, extConfig) {
    var _a, _b;
    const name = 'ios';
    const podPath = determineCocoapodPath();
    const platformDir = (_b = (_a = extConfig.ios) === null || _a === void 0 ? void 0 : _a.path) !== null && _b !== void 0 ? _b : 'ios';
    const platformDirAbs = path_1.resolve(rootDir, platformDir);
    const nativeProjectDir = 'App';
    const nativeTargetDir = `${nativeProjectDir}/App`;
    const webDir = `${nativeProjectDir}/public`;
    const cordovaPluginsDir = 'capacitor-cordova-ios-plugins';
    return {
        name,
        minVersion: '12.0',
        platformDir,
        platformDirAbs,
        cordovaPluginsDir,
        cordovaPluginsDirAbs: path_1.resolve(platformDirAbs, cordovaPluginsDir),
        nativeProjectDir,
        nativeProjectDirAbs: path_1.resolve(platformDirAbs, nativeProjectDir),
        nativeTargetDir,
        nativeTargetDirAbs: path_1.resolve(platformDirAbs, nativeTargetDir),
        webDir,
        webDirAbs: path_1.resolve(platformDirAbs, webDir),
        podPath,
    };
}
async function loadWebConfig(rootDir, webDir) {
    const platformDir = webDir;
    const platformDirAbs = path_1.resolve(rootDir, platformDir);
    return {
        name: 'web',
        platformDir,
        platformDirAbs,
    };
}
function determineOS(os) {
    switch (os) {
        case 'darwin':
            return "mac" /* Mac */;
        case 'win32':
            return "windows" /* Windows */;
        case 'linux':
            return "linux" /* Linux */;
    }
    return "unknown" /* Unknown */;
}
async function determineAndroidStudioPath(os) {
    if (process.env.CAPACITOR_ANDROID_STUDIO_PATH) {
        return process.env.CAPACITOR_ANDROID_STUDIO_PATH;
    }
    switch (os) {
        case "mac" /* Mac */:
            return '/Applications/Android Studio.app';
        case "windows" /* Windows */: {
            const { runCommand } = await Promise.resolve().then(() => tslib_1.__importStar(require('./util/subprocess')));
            let p = 'C:\\Program Files\\Android\\Android Studio\\bin\\studio64.exe';
            try {
                if (!(await utils_fs_1.pathExists(p))) {
                    let commandResult = await runCommand('REG', [
                        'QUERY',
                        'HKEY_LOCAL_MACHINE\\SOFTWARE\\Android Studio',
                        '/v',
                        'Path',
                    ]);
                    commandResult = commandResult.replace(/(\r\n|\n|\r)/gm, '');
                    const i = commandResult.indexOf('REG_SZ');
                    if (i > 0) {
                        p = commandResult.substring(i + 6).trim() + '\\bin\\studio64.exe';
                    }
                }
            }
            catch (e) {
                debug(`Error checking registry for Android Studio path: %O`, e);
                break;
            }
            return p;
        }
        case "linux" /* Linux */:
            return '/usr/local/android-studio/bin/studio.sh';
    }
    return '';
}
function determineCocoapodPath() {
    if (process.env.CAPACITOR_COCOAPODS_PATH) {
        return process.env.CAPACITOR_COCOAPODS_PATH;
    }
    return 'pod';
}
